@@ -178,7 +178,26 @@ class AutosubmitConfig(object):
          :rtype: string
          """
         return wrapper.get('EXPORT', self.experiment_data["WRAPPERS"].get("EXPORT",""))
-
+    def get_project_submodules_depth(self):
+        """
+        Returns the max depth of submodule at the moment of cloning
+        Default is -1 (no limit)
+        :return: depth
+        :rtype: list
+        """
+        git_data= self.experiment_data.get("GIT",{})
+        unparsed_depth = git_data.get('PROJECT_SUBMODULES_DEPTH', "-1")
+        if "[" in unparsed_depth and "]" in unparsed_depth:
+            unparsed_depth = unparsed_depth.strip("[]")
+            depth = [int(x) for x in unparsed_depth.split(",")]
+        else:
+            try:
+                depth = int(unparsed_depth)
+                depth = [depth]
+            except:
+                Log.warning("PROJECT_SUBMODULES_DEPTH is not an integer neither a int. Using default value -1")
+                depth = []
+        return depth
     def get_full_config_as_json(self):
         """
         Return config as json object
@@ -580,7 +599,17 @@ class AutosubmitConfig(object):
             if type(custom_directives) != str:
                 data_fixed["JOBS"][job]["CUSTOM_DIRECTIVES"] = str(custom_directives)
             data_fixed["JOBS"][job]["DEPENDENCIES"] = dependencies
-
+            files = job_data.get("FILE","")
+            if ',' in files:
+                files = files.split(",")
+            elif ' ' in files:
+                files = files.split(" ")
+            else:
+                files = [files]
+            data_fixed["JOBS"][job]["FILE"] = files[0]
+            data_fixed["JOBS"][job]["ADDITIONAL_FILES"] = []
+            for file in files[1:]:
+                data_fixed["JOBS"][job]["ADDITIONAL_FILES"].append(file)
 
         return data_fixed
     def unify_conf(self):
@@ -735,7 +764,7 @@ class AutosubmitConfig(object):
 
 
     def check_mandatory_conf_files(self):
-        self.unify_conf()
+        #self.unify_conf()
         self.check_expdef_conf()
         self.check_platforms_conf()
         self.check_jobs_conf()
@@ -1079,6 +1108,7 @@ class AutosubmitConfig(object):
         Creates parser objects for configuration files
         """
         any_file_changed = False
+        modified = False
         # check if original_files has been edited
         for config_file in range(0,len(self._original_parser_files)):
             try:
@@ -1130,16 +1160,31 @@ class AutosubmitConfig(object):
                         self._platforms_parser.data = {}
                     default_section = self._exp_parser.data.get("DEFAULT",None)
                     default_path = Path(self.basic_config.LOCAL_ROOT_DIR) / self.expid
-                    custom_folder_path = default_path / "conf"
+                    custom_folder_path = default_path / "conf" / "custom_conf"
+                    if not custom_folder_path.exists():
+                        try:
+                            os.mkdir(custom_folder_path)
+                            os.chmod(custom_folder_path, 0o770)
+                        except:
+                            pass
                     if default_section is not None and len(str(default_section)) > 0:
-                        custom_folder_path = Path(re.sub('%(?<!%%)' + "ROOTDIR" + '%(?!%%)', str(default_path), str(custom_folder_path), flags=re.I))
-                        default_section["CUSTOM_CONFIG_DIR"] = str(custom_folder_path)
+                        default_section["CUSTOM_CONFIG"] = default_section.get("CUSTOM_CONFIG",str(custom_folder_path))
+                        custom_config = re.sub('%(?<!%%)' + "ROOTDIR" + '%(?!%%)', str(default_path), default_section["CUSTOM_CONFIG"], flags=re.I)
                     else:
-                        custom_folder_path = default_path / "conf"
+                        custom_config = str(default_path / "conf" / "custom_conf")
                     self._custom_parser_files = []
                     self._custom_parser_files_modtime = []
-
-                    for f in custom_folder_path.rglob("*.yml"):
+                    # Parse folder. Should always exists as first item.
+                    if ',' in custom_config:
+                        custom_config = custom_config.split(',')
+                    elif ' ' in custom_config:
+                        custom_config = custom_config.split(' ')
+                    else:
+                        custom_config = [custom_config]
+                    custom_folder_path = [x for x in Path(custom_config[0]).rglob("*.yml") ] + [x for x in Path(custom_config[0]).rglob("*.yaml") ]
+                    if len(custom_config) > 1:
+                        custom_folder_path.extend([Path(x) for x in custom_config[1:] ])
+                    for f in custom_folder_path:
                         if not (self._proj_parser_file.exists() and f.samefile(self._proj_parser_file)) \
                                 and not (self._jobs_parser_file.exists() and f.samefile(self._jobs_parser_file)) \
                                 and not (self._platforms_parser_file.exists() and f.samefile(self._platforms_parser_file)) \
@@ -2082,6 +2127,37 @@ class AutosubmitConfig(object):
         commit = self.get_git_project_commit()
         return origin_exists and ( (branch is not None and len(str(branch)) > 0) or ( commit is not None and len(str(commit)) > 0))
 
+    def parse_githooks(self):
+        """
+        Parse githooks section in configuration file
+
+        :return: dictionary with githooks configuration
+        :rtype: dict
+        """
+        proj_dir = os.path.join(
+            BasicConfig.LOCAL_ROOT_DIR, self.expid, BasicConfig.LOCAL_PROJ_DIR)
+        # get project_name
+        project_name = str(self.get_project_destination())
+
+        # get githook files from proj_dir
+        githook_files = [os.path.join(os.path.join(os.path.join(proj_dir, project_name), ".githooks"), f) for f in
+                         os.listdir(os.path.join(os.path.join(proj_dir, project_name), ".githooks"))]
+        parameters = self.load_parameters()
+
+        # find all '%(?<!%%)\w+%(?!%%)' in githook files
+        for githook_file in githook_files:
+            f_name, ext = os.path.splitext(githook_file)
+            if ext == ".tmpl":
+                with open(githook_file, 'r') as f:
+                    content = f.read()
+                matches = re.findall('%(?<!%%)\w+%(?!%%)', content)
+                for match in matches:
+                    # replace all '%(?<!%%)\w+%(?!%%)' with parameters value
+                    content = content.replace(match, parameters.get(match[1:-1], ""))
+                with open(f_name, 'w') as f:
+                    f.write(content)
+                    os.chmod(f_name, 0o750)
+        pass
     @staticmethod
     def get_parser(parser_factory, file_path):
         """
@@ -2128,3 +2204,22 @@ class AutosubmitConfig(object):
                     "{}\n This file and the correctness of its content are necessary.".format(str(exp)))
         return parser
 
+
+    @staticmethod
+    def parse_placeholders(content, parameters):
+        """
+        Parse placeholders in content
+
+        :param content: content to be parsed
+        :type content: str
+        :param parameters: parameters to be used in parsing
+        :type parameters: dict
+        :return: parsed content
+        :rtype: str
+        """
+        matches = re.findall('%(?<!%%)[a-zA-Z0-9_.]+%(?!%%)', content,flags=re.I)
+        for match in matches:
+            # replace all '%(?<!%%)\w+%(?!%%)' with parameters value
+            content = content.replace(match, parameters.get(match[1:-1], ""))
+        return content
+
